      PROGRAM McMAILLE
*
*     Version 2.0
*
*     MAILLE in french = CELL in english 
*     MC for Monte Carlo
*
*
************************************************************************ 
*
*     A Monte Carlo and grid search code for indexing
*     by Le Bail fitting to a pseudo powder pattern built up from the 
*        2-theta angles and intensities from a peak hunting program, 
*        applying Gaussian peak shapes, with Caglioti law (U, V, W) 
*
*     For more details see the documentation at 
*                   http://www.cristal.org/sdpd/McMaille/ 
*             or    http://sdpd.univ-lemans.fr/sdpd/McMaille/ 
*
*
*                   by A. Le Bail - September 2002 for version 0.9
*                                   as well as for versions 1.0 and 2.0             
*                        alb@cristal.org 
*                        http://www.cristal.org/ 
*
*                        Rsidence Cristal - Appt 213 
*                        2, rue de Gasperi 
*                        72100 Le Mans 
*                        FRANCE
*
*    Version 0.9 : cubic only
*            1.0 : hexagonal/trigonal/rhombohedral, tetragonal, 
*                  orthorhombic added, plus .ckm and .prf files
*            2.0 : monoclinic and triclinic added in MC
*                  but not in grid search (too long)
*
*
************************************************************************ 
*
*    Copyright (C) 2002 Armel Le Bail 
*
* This program is free software; you can redistribute it and/or 
* modify it under the terms of the GNU General Public License 
* as published by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful, 
* but WITHOUT ANY WARRANTY; without even the implied warranty of 
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License 
* along with this program; if not, write to the Free Software 
* Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*
*********************************************************************** 
*
*
*
      INCLUDE 'esp0.inc'
      INCLUDE 'esp2.inc'
	CHARACTER*79 BUFFER
      CHARACTER*24 FILE
      CHARACTER*24 TEMPO
	CHARACTER*1 FEND
      CHARACTER STRSYM*50,SELECT1*80,PUFFER*7,VERSION*8
	INTEGER(4) PROGRESS
      LOGICAL QEX
      LOGICAL FIRST,LOOPING,ACCEPTABLE
      PARAMETER(VERSION='1.0')
C
C
      DIMENSION TH2(N_HKL),FOBS(N_HKL),FCAL(N_HKL),D(N_HKL)
      DIMENSION IHH(3,N_HKL),JHH(3,N_HKL),HH(3),NSYS(6),PSTARTB(6)
      DIMENSION CELPRE(6),CELLN(6),AL(3,3),RCELLN(6),CELL(6),CELOLD(6)
      DIMENSION BPAR(6),CEL(6,1000),RP(1000),VGC(1000),LL(1000)
	DIMENSION IFI(1000),IREFS(1000),TEXT(20)
C
C
      PI=114.59156  ! 360./3.1415926
      N1=1
      N2=2
C
      PRINT 1
1     FORMAT('  Entry file (no extension) ??',$)
      READ 2,FILE
      WRITE(*,*)FILE
2     FORMAT(A20)
      LFILE=LEN(FILE)
      DO WHILE(FILE(LFILE:LFILE).EQ.' ')
      LFILE=LFILE-1
      ENDDO
      LC0=19
      LCA=21
      TEMPO=FILE(1:LFILE)//'.inp'
      INQUIRE(FILE=TEMPO,EXIST=QEX)
      IF(.NOT.QEX) GO TO 3
      CALL FILEDEL(21,TEMPO)
3     CALL OPEN_WRITE1(21,TEMPO)
      TEMPO=FILE(1:LFILE)//'.dat'
      CALL OPEN_READ1(19,TEMPO)
4     READ(19,5,ERR=6)SELECT1
      IF(SELECT1(1:1).NE.'!')WRITE(21,5)SELECT1
      GO TO 4
5     FORMAT(A80)
6     CLOSE(LC0)
      CLOSE(LCA)
      TEMPO=FILE(1:LFILE)//'.inp'
      CALL OPEN_READ1(21,TEMPO)
      LPR=20
      TEMPO=FILE(1:LFILE)//'.imp'
      INQUIRE(FILE=TEMPO,EXIST=QEX)
      IF(.NOT.QEX) GO TO 7
      CALL FILEDEL(20,TEMPO)
7     CALL OPEN_WRITE1(20,TEMPO)
	WRITE(BUFFER,*)'McMaille version ',VERSION
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
	WRITE(BUFFER,*)'Data file : ',FILE(1:LFILE)
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      WRITE (20,3000) VERSION,FILE(1:LFILE)
      CALL DATN
C
C.....READ PROBLEM IDENTIFICATION
C
      READ(LCA,8,ERR=3500)(TEXT(I),I=1,20)
8     FORMAT(20A4)
      WRITE(20,*)
      WRITE(20,8)(TEXT(I),I=1,20)
      WRITE(20,*)
C
C.....READ wavelength and type of calculation
C     SLABDA = wavelength
C     ZERO   = zeropoint to be added at thr beginning
C     NGRID  if = 1  : grid cell generation
C            if = 0  : Monte Carlo cell generation
C            if = 2  : both
C
      READ(LCA,*,ERR=3501)SLABDA,ZERO,NGRID 
C
C.....READ codes for search in crystalline systems
C
C    NSYS(n)
C     n
C     1   Cubic
C     2   Hexagonal/trigonal/Rhombohedral
C     3   Tetragonal
C     4   Orthorhombic
C     5   Monoclinic
C     6   Triclinic
C
C     if NSYS(n)=0 : no search
C     if NSYS(n)=1 : search
C     if NSYS(2)=2 : search in rhombohedral
C
      READ(LCA,*,ERR=3502)(NSYS(I),I=1,6)
C
C.....Read the U,V,W,STEPN
C
      READ(LCA,*,ERR=3503)U,V,W,STEPN
      IF(STEPN.LT.3.)STEPN=3.
      IF(STEPN.GT.5.)STEPN=5.
C
C.....Some printing
C
      WRITE(20,*)
      WRITE(20,9)SLABDA
9     FORMAT(' Wavelength : ',F9.5/)
      WRITE(20,*)
      IF(NGRID.EQ.0)WRITE(20,*)'Monte Carlo cell generation'
      IF(NGRID.EQ.1)WRITE(20,*)'Grid cell generation'
      IF(NGRID.EQ.2)WRITE(20,*)'Both searches - Monte Carlo AND grid'
	WRITE(20,*)
      WRITE(20,10)U,V,W,STEPN
10    FORMAT(' with U, V, W, STEPN  = ',4F9.4) 
C
C.....READ min and max parameters, volume, Rp
C
      READ(LCA,*,ERR=3504)PMIN,PMAX,VMIN,VMAX,RMI,RMAX
	WRITE(20,*)
	WRITE(20,*)' Min and Max a,b,c, cell parameters ',PMIN,PMAX
	WRITE(20,*)
	WRITE(20,*)' Min and Max volumes ',VMIN,VMAX
	WRITE(20,*)
	WRITE(20,*)' Min and Max Rp ',RMI,RMAX
C
C.....According to NGRID, read either grid steps
C                              or Monte Carlo parameters 
C                              or both
      IF(NGRID.EQ.0)GO TO 12
      IF(NGRID.EQ.1)GO TO 11
      IF(NGRID.EQ.2)GO TO 11
      WRITE(20,*)
      WRITE(20,*)' UNKNOWN NGRID PARAMETER : STOP'
      STOP
11    CONTINUE
C
C.....READ grid steps and RMAX
C     SPAR = step on cell parameters
C     SANG = step on angles
C     RMAX : refine if Rp < RMAX
C
      READ(LCA,*,ERR=3505)SPAR,SANG
	WRITE(20,*)
	WRITE(20,*)' Steps on (a,b,c) and angles ',SPAR,SANG
      IF(NGRID.EQ.2)GO TO 12
      GO TO 13
12    CONTINUE
C
C  Continue up to ntimelin tests
C  Save parameters if Rp < Rmax
C  Make NRUNS times those ntimelim tests
C
      READ (LCA,*,ERR=3506)NTIMELIM,NRUNS
	WRITE(20,*)
	WRITE(20,*)' N of MC events, N of runs ',NTIMELIM,NRUNS
C
13    CONTINUE
C
C... Make a WARNING about the expected CPU time...
C
      TTEST=0
      TT=NTIMELIM*NRUNS
	IF(NGRID.EQ.0)GO TO 78     
      TTGP=(PMAX-PMIN)/SPAR+1.
	TTGA=(130.-90.)/SANG+1.
78    DO 80 I=1,6
	IF(NGRID.EQ.1)GO TO 79
C  SUM on Monte Carlo tests
      IF(NSYS(I).EQ.0)GO TO 79
      IF(I.EQ.1)TTEST=TTEST+TT
      IF(I.EQ.2)TTEST=TTEST+TT
      IF(I.EQ.3)TTEST=TTEST+TT
      IF(I.EQ.4)TTEST=TTEST+TT
      IF(I.EQ.5)TTEST=TTEST+TT
      IF(I.EQ.6)TTEST=TTEST+TT
79    IF(NSYS(I).EQ.0)GO TO 80
      IF(NGRID.EQ.0)GO TO 80
C  SUM on grid tests
      IF(I.EQ.1)TTEST=TTEST+TTGP
      IF(I.EQ.2)TTEST=TTEST+TTGP**2
      IF(I.EQ.3)TTEST=TTEST+TTGP**2
      IF(I.EQ.4)TTEST=TTEST+TTGP**3
      IF(I.EQ.5)TTEST=TTEST+TTGP**3*TTGA
      IF(I.EQ.6)TTEST=TTEST+TTGP**3*TTGA**3
80    CONTINUE
C
C  ESTIMATED CPU TIMU on the 2.4 GHz basis of 
C                     1000 tests/second
C
      TIMES=TTEST/1000.
	TIMEH=TTEST/3600000.
	TIMED=TIMEH/24.
	TIMEY=TIMED/365.
      WRITE(20,*)
	WRITE(20,*)'WARNING - WARNING - WARNING - WARNING - WARNING'
      WRITE(20,*)'EXPECTED total number of tests ',TTEST
	WRITE(20,*)'EXPECTED TOTAL CPU TIME FOR A 2.4GHz:'
	WRITE(20,*)'Seconds: ',TIMES
	WRITE(20,*)'Hours:   ',TIMEH
	WRITE(20,*)'Days:    ',TIMED
	WRITE(20,*)'Years:   ',TIMEY
	WRITE(20,*)'If you do not want to continue : Ctrl+C would be better'
	WRITE(20,*)
      PRINT *
	PRINT *,'WARNING - WARNING - WARNING - WARNING - WARNING'
      PRINT *,'EXPECTED total number of tests ',TTEST
	PRINT *,'EXPECTED TOTAL CPU TIME FOR A 2.4GHz:'
	PRINT *,'Seconds: ',TIMES
	PRINT *,'Hours:   ',TIMEH
	PRINT *,'Days:    ',TIMED
	PRINT *,'Years:   ',TIMEY
	PRINT *,'If you do not want to continue : Ctrl+C would be better'
	PRINT *
C
C.....And now, read couples of 2-theta and I values
C
      SUM_F=0.
      NDAT=1
14    READ(LCA,*,END=15)TH2(NDAT),FOBS(NDAT)
      IF(TH2(NDAT).GE.180.)GO TO 3508
C
C     Addition of the Zeropoint
C
      TH2(NDAT)=TH2(NDAT)+ZERO
      SUM_F=SUM_F+FOBS(NDAT)
      NDAT=NDAT+1
      IF(NDAT.GT.100)WRITE(20,*)'Max data = 100 !'
      IF(NDAT.GT.100)STOP
      GO TO 14
15    CONTINUE
      NDAT=NDAT-1
C
CCC Normalization of FOBS to 50. as a mean, arbitrarily
C
      XNHKL=NDAT
      HMIN=100.
      DO 16 I=1,NDAT
16    FOBS(I)=FOBS(I)/SUM_F*XNHKL*50.
      SUM_F=XNHKL*50.
      NHKL=NDAT
      FLOG =4.*ALOG(2.)
      SLOG =1./SQRT(FLOG)
      SUM_FS=0.
C
C.....END OF DATA READING
C
      CLOSE (LCA,STATUS='DELETE')
C
C.....START TO GENERATE THE "OBSERVED" PROFILE
C
C   Sum on observed peak positions
C
      DO 17 NM=1,NHKL
C
      DD=SLABDA/(2.*SIN(TH2(NM)/PI))
      SSS=1/DD**2
C
      SSNN=0.25*SSS
C
C     Needs to calculate halfwidths and positions 
C
      D(NM)=SQRT(1./SSS)
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
17    CONTINUE
      DMIN=D(NHKL)
C	HMIN=HMIN*SLOG
C
C  Step and positions
C  The minimum step is FWHMmin/STEPN
C
      STEP=HMIN/STEPN
      ASTEP=STEP*1000.
      ISTEP=ASTEP
      ASTEP=ISTEP
      STEP=ASTEP/1000.
      POS(1)=STEP
      PO=STEP
      DO 18 I=2,16000
      TANTH=TAN(POS(I-1)/PI)
      UU=U*TANTH*TANTH+V*TANTH+W
      IF(UU.LE.0.)GO TO 19
      PO=PO+STEP
      IF(PO.GT.160.)GO TO 19
18    POS(I)=PO
19    NPTS=I-1
      CALL PROFILE0(NHKL,FOBS)
      SUM_Y=0.
      N1=1
      DO 20 I=N1,N2
20    SUM_Y=SUM_Y+YOBS(I)
C
C  Output of some data
C
      WRITE(20,*)
      WRITE(20,*)'    2-THETA    Iobs        D         HW'
      DO 21 NM=1,NHKL
      WRITE(20,22)THETA(NM),FOBS(NM),D(NM),HW(NM)
21    CONTINUE
22    FORMAT(2X,6F10.3)
      WRITE(20,*)
C
C...  Various starting values initialized
C
C      DMIN=SLABDA/(2.*SIN(POS(N2)/PI))
      WRITE(20,*)
      WRITE(20,*)'   Dmin = ',DMIN
      DELTA=(PMAX-PMIN)/2.
      PSTART=DELTA+PMIN
      DELTC=(120.-90.)/2.
      ASTART=DELTC+90.
      NTRIEDB=0
      RMAX0=RMAX
	RMIN=RMAX
C   DELTAB = zone explored in angstrom around a good cell
	DELTAB=0.02
C   DELTAD = zone explored in 1/100 of degrees around a good cell
	DELTAD=0.02
      DMIN=DMIN-DELTAD
C   NCYCLES ) number of Le Bail fits for that exploration
	NCYCLES=100
C  IGC = number of retained cells, IGM = max IGC
      IGC=0
	IGM=1000
C  NLB = Number of Le Bail fits after the first one
      NLB=1
C  Note that thre is also a last one
C       so that if NLB=1, the Le bail fit number is 3
      CALL DATN
C
C
C.....AND NOW : Generate the cell, either by Monte Carlo
C               or by grid search
C
      IF(NGRID.EQ.1)GO TO 700
C
C...  Cell generation by Monte Carlo
C

      IF(NSYS(1).EQ.0)GO TO 200
C
C    Cubic case
C
	DO 199 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=1
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in cubic, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in cubic, run :',NRUN
	PRINT *,'     Rp     Trial number    a     V'
C
C     READ hkl Miller indices in cub.hkl
C
      TEMPO='cub.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 101 I=1,NHKL0
101   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
      NTRIED=0
C
C...  here starts the loop
C
102   CONTINUE
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=NTRIED+1
      GO TO 104
103   CELPRE(1)=PSTARTB(1)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
104   CONTINUE
      CELPRE(2)=CELPRE(1)
      CELPRE(3)=CELPRE(1)
      DO 105 I=1,3
      DO 105 J=1,3
105   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 116
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 102
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 109 I=1,NHKL0
      DO 106 KK=1,3
106   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 107 J=1,3
      DO 107 K=J,3
107   X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 109
      JH=JH+1
      DO 108 KK=1,3
108   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
109   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 110 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
110   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 111 J=1,NHKL
      FCAL(J)=FOBS(J)
111   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 112 K=1,NLB
      CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 112 J=1,NHKL
      FCAL(J)=FOBS(J)
112   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 114
      IF(DIFF.GE.RMAX) GO TO 115
114   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
	A=CELPRE(1)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 103
      WRITE(20,*)RMAX,NTRIED,A,V2
      PRINT *,RMAX,NTRIED,A,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=A
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
115   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 116
      GO TO 102
116   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 199
      WRITE(20,*)
	WRITE(20,*)'Best result : a=',BPAR(1),'V=',V3,' Rp=',RMIN
	PRINT *,'Best result : a=',BPAR(1),'V=',V3,' Rp=',RMIN
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
199   CONTINUE
200   IF(NSYS(2).EQ.0)GO TO 300
C
C    Hexagonal case
C
C
	DO 299 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=2
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=120.
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in hexagonal, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a      c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in hexagonal, run :',NRUN
	PRINT *,'     Rp     Trial number    a      c     V'
C
C     READ hkl Miller indices in hex.hkl
C
      IF(NSYS(2).EQ.2)GO TO 260
      TEMPO='hex.hkl'
	GO TO 261
260   TEMPO='rho.hkl'
	IFILE=7
261   CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 201 I=1,NHKL0
201   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
	CELPRE(2)=CELPRE(1)
      CELPRE(3)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=0
C
202   CONTINUE
C
C     Which parameter to vary ? a or c ?
C
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=NTRIED+1
      GO TO 204
203   CONTINUE
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
204   CONTINUE
      CELPRE(2)=CELPRE(1)
      DO 205 I=1,3
      DO 205 J=1,3
205   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 216
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 202
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 209 I=1,NHKL0
      DO 206 KK=1,3
206   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 207 J=1,3
      DO 207 K=J,3
207   X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 209
      JH=JH+1
      DO 208 KK=1,3
208   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
209   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 210 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
210   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 211 J=1,NHKL
      FCAL(J)=FOBS(J)
211   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 212 K=1,NLB
      CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 212 J=1,NHKL
      FCAL(J)=FOBS(J)
212   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 214
      IF(DIFF.GE.RMAX) GO TO 215
214   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 203
      WRITE(20,*)RMAX,NTRIED,A,C,V2
      PRINT *,RMAX,NTRIED,A,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=120.
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
215   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 216
      GO TO 202
216   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 299
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
299   CONTINUE
300   IF(NSYS(3).EQ.0)GO TO 400
C
C    Tetragonal case
C
C
	DO 399 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=3
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in tetragonal, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a      c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in tetragonal, run :',NRUN
	PRINT *,'     Rp     Trial number    a      c     V'
C
C     READ hkl Miller indices in tet.hkl
C
      TEMPO='tet.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 301 I=1,NHKL0
301   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
	CELPRE(2)=CELPRE(1)
      CELPRE(3)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=0
C
302   CONTINUE
C
C     Which parameter to vary ? a or c ?
C
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=NTRIED+1
      GO TO 304
303   CONTINUE
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
304   CONTINUE
      CELPRE(2)=CELPRE(1)
      DO 305 I=1,3
      DO 305 J=1,3
305   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 316
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 302
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 309 I=1,NHKL0
      DO 306 KK=1,3
306   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 307 J=1,3
      DO 307 K=J,3
307   X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 309
      JH=JH+1
      DO 308 KK=1,3
308   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
309   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 310 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
310   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 311 J=1,NHKL
      FCAL(J)=FOBS(J)
311   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 312 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 312 J=1,NHKL
      FCAL(J)=FOBS(J)
312   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 314
      IF(DIFF.GE.RMAX) GO TO 315
314   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 303
      WRITE(20,*)RMAX,NTRIED,A,C,V2
      PRINT *,RMAX,NTRIED,A,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
      IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
315   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 316
      GO TO 302
316   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 399
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
399   CONTINUE
400   IF(NSYS(4).EQ.0)GO TO 500
C
C    Orthorhombic case
C
C
	DO 499 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=4
      NCYCLES=400
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in orthorhombic, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a   b   c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in orthorhombic, run :',NRUN
	PRINT *,'     Rp     Trial number    a   b   c     V'
C
C     READ hkl Miller indices in ort.hkl
C
      TEMPO='ort.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 401 I=1,NHKL0
401   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(2)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(3)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=0
C
402   CONTINUE
C
C     Which parameter to vary ? a or b or c ?
C
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.33333)I=1
      IF(X.GE.0.33333.AND.X.LT.0.66666)I=2
      IF(X.GE.0.66666.AND.X.LE.1.)I=3
      CELPRE(I)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      NTRIED=NTRIED+1
      GO TO 404
403   CONTINUE
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.33333)I=1
      IF(X.GE.0.33333.AND.X.LT.0.66666)I=2
      IF(X.GE.0.66666.AND.X.LT.1.)I=3
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
404   CONTINUE
      DO 405 I=1,3
      DO 405 J=1,3
405   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 416
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 402
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 409 I=1,NHKL0
      DO 406 KK=1,3
406   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 407 J=1,3
      DO 407 K=J,3
407   X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 409
      JH=JH+1
      DO 408 KK=1,3
408   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
409   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 410 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
410   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 411 J=1,NHKL
      FCAL(J)=FOBS(J)
411   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 412 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 412 J=1,NHKL
      FCAL(J)=FOBS(J)
412   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 414
      IF(DIFF.GE.RMAX) GO TO 415
414   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	B=CELPRE(2)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	BPAR(2)=B
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(2)=CELPRE(2)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 403
      WRITE(20,*)RMAX,NTRIED,A,B,C,V2
      PRINT *,RMAX,NTRIED,A,B,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=B
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
415   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 416
      GO TO 402
416   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 499
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : b = ',BPAR(2)
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : b = ',BPAR(2)
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
499   CONTINUE
500   IF(NSYS(5).EQ.0)GO TO 600
C
C    Monoclinic case
C
C
	DO 599 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=5
      NCYCLES=1000
      CELPRE(4)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in monoclinic, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a   b   c  bet   V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in monoclinic, run :',NRUN
	PRINT *,'     Rp     Trial number    a   b   c  bet   V'
C
C     READ hkl Miller indices in mon.hkl
C
      TEMPO='mon.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 501 I=1,NHKL0
501   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(2)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(3)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(5)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      NTRIED=0
C
502   CONTINUE
C
C     Which parameter to vary ? a or b or c or beta ?
C
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.25)I=1
      IF(X.GE.0.25.AND.X.LT.0.5)I=2
      IF(X.GE.0.5.AND.X.LT.0.75)I=3
      IF(X.GE.0.75.AND.X.LE.1.0)I=5
      IF(I.NE.5)THEN
      CELPRE(I)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      ELSE
      CELPRE(I)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      ENDIF
      NTRIED=NTRIED+1
      GO TO 504
503   CONTINUE
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.25)I=1
      IF(X.GE.0.25.AND.X.LT.0.5)I=2
      IF(X.GE.0.5.AND.X.LT.0.75)I=3
      IF(X.GE.0.75.AND.X.LE.1.0)I=5
      IF(I.NE.5)THEN
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      ELSE
      CELPRE(I)=PSTARTB(I)+(DELTAD*(RAN(ISEED)-0.5)*2.)
      ENDIF
      NTRIEDB=NTRIEDB+1
504   CONTINUE
      DO 505 I=1,3
      DO 505 J=1,3
505   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 516
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 502
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 509 I=1,NHKL0
      DO 506 KK=1,3
506   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 507 J=1,3
      DO 507 K=J,3
507   X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 509
      JH=JH+1
      DO 508 KK=1,3
508   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
509   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 510 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
510   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 511 J=1,NHKL
      FCAL(J)=FOBS(J)
511   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 512 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 512 J=1,NHKL
      FCAL(J)=FOBS(J)
512   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 514
      IF(DIFF.GE.RMAX) GO TO 515
514   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	B=CELPRE(2)
	C=CELPRE(3)
	BET=CELPRE(5)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	BPAR(2)=B
      BPAR(3)=C
      BPAR(5)=BET
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(2)=CELPRE(2)
      PSTARTB(3)=CELPRE(3)
      PSTARTB(5)=CELPRE(5)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 503
      WRITE(20,*)RMAX,NTRIED,A,B,C,BET,V2
      PRINT *,RMAX,NTRIED,A,B,C,BET,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=B
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=BET
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
515   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 516
      GO TO 502
516   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 599
      WRITE(20,*)
	WRITE(20,*)'Best result : a =    ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : b =    ',BPAR(2)
	WRITE(20,*)'Best result : c =    ',BPAR(3)
	WRITE(20,*)'Best result : beta = ',BPAR(5),'  V = ',V3
	PRINT *,'Best result : a =    ',BPAR(1),'  Rp = ',RMIN
	PRINT *,'Best result : b =    ',BPAR(2)
	PRINT *,'Best result : c =    ',BPAR(3)
	PRINT *,'Best result : beta = ',BPAR(5),'  V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
599   CONTINUE
600   IF(NSYS(6).EQ.0)GO TO 700
C
C    Triclinic case
C
C
	DO 699 NRUN=1,NRUNS
*------------------------------------------------------------------------- 
*     Initialisation
*
      CALL ESP_INIT(ISEED,TIME_START)
*
*-------------------------------------------------------------------------
      IFILE=6
      NCYCLES=2000
	WRITE(20,*)
	WRITE(20,*)'Monte Carlo search :'
	WRITE(20,*)
	WRITE(20,*)'Results in triclinic, run :',NRUN
	WRITE(20,*)'     Rp     Trial number    a   b   c  alp bet gam  V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Monte Carlo search :'
	PRINT *
	PRINT *,'Results in triclinic, run :',NRUN
	PRINT *,'     Rp     Trial number    a   b   c  alp bet gam  V'
C
C     READ hkl Miller indices in tri.hkl
C
      TEMPO='tri.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 601 I=1,NHKL0
601   READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(2)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(3)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      CELPRE(4)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      CELPRE(5)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      CELPRE(6)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      NTRIED=0
C
602   CONTINUE
C
C     Which parameter to vary ? a or b or c or alpha or beta or gamma ?
C
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.16666)I=1
      IF(X.GE.0.16666.AND.X.LT.0.33333)I=2
      IF(X.GE.0.33333.AND.X.LT.0.5)I=3
      IF(X.GE.0.5.AND.X.LT.0.66666)I=4
      IF(X.GE.0.66666.AND.X.LT.0.83333)I=5
      IF(X.GE.0.83333.AND.X.LE.1.0)I=6
      IF(I.NE.4.AND.I.NE.5.AND.I.NE.6)THEN
      CELPRE(I)=PSTART+(DELTA*(RAN(ISEED)-0.5)*2.)
      ELSE
      CELPRE(I)=ASTART+(DELTC*(RAN(ISEED)-0.5)*2.)
      ENDIF
      ANG=CELPRE(4)+CELPRE(5)+CELPRE(6)
      IF(ANG.GE.360.)GO TO 602
      NTRIED=NTRIED+1
      GO TO 604
603   CONTINUE
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.16666)I=1
      IF(X.GE.0.16666.AND.X.LT.0.33333)I=2
      IF(X.GE.0.33333.AND.X.LT.0.5)I=3
      IF(X.GE.0.5.AND.X.LT.0.66666)I=4
      IF(X.GE.0.66666.AND.X.LT.0.83333)I=5
      IF(X.GE.0.83333.AND.X.LE.1.0)I=6
      IF(I.NE.4.AND.I.NE.5.AND.I.NE.6)THEN
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      ELSE
      CELPRE(I)=PSTARTB(I)+(DELTAD*(RAN(ISEED)-0.5)*2.)
      ENDIF
      ANG=CELPRE(4)+CELPRE(5)+CELPRE(6)
      IF(ANG.GE.360.)GO TO 603
      NTRIEDB=NTRIEDB+1
604   CONTINUE
      DO 605 I=1,3
      DO 605 J=1,3
605   AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(NTRIED.GT.NTIMELIM)GO TO 616
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 602
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 609 I=1,NHKL0
      DO 606 KK=1,3
606   HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 607 J=1,3
      DO 607 K=J,3
607   X=AL(J,K)*HH(J)*HH(K)+X
      IF(X.LT.0.)PRINT *,' X = ',X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 609
      JH=JH+1
      DO 608 KK=1,3
608   JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
609   CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 610 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
610   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 611 J=1,NHKL
      FCAL(J)=FOBS(J)
611   CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 612 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 612 J=1,NHKL
      FCAL(J)=FOBS(J)
612   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 614
      IF(DIFF.GE.RMAX) GO TO 615
614   IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	B=CELPRE(2)
	C=CELPRE(3)
	ALP=CELPRE(4)
	BET=CELPRE(5)
	GAM=CELPRE(6)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	BPAR(2)=B
      BPAR(3)=C
      BPAR(4)=ALP
      BPAR(5)=BET
      BPAR(6)=GAM
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(2)=CELPRE(2)
      PSTARTB(3)=CELPRE(3)
      PSTARTB(4)=CELPRE(4)
      PSTARTB(5)=CELPRE(5)
      PSTARTB(6)=CELPRE(6)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 603
      WRITE(20,*)RMAX,NTRIED,A,B,C,ALP,BET,GAM,V2
      PRINT *,RMAX,NTRIED,A,B,C,ALP,BET,GAM,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=B
      CEL(3,IGC)=C
      CEL(4,IGC)=ALP
      CEL(5,IGC)=BET
      CEL(6,IGC)=GAM
      NTRIEDB=0
      RMAX=RMAX0
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
615   CONTINUE
C
C... Stop if max limit of Monte Carlo tests outpassed
C
      IF(NTRIED.GT.NTIMELIM)GO TO 616
      GO TO 602
616   CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 699
      WRITE(20,*)
	WRITE(20,*)'Best result : a =    ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : b =    ',BPAR(2)
	WRITE(20,*)'Best result : c =    ',BPAR(3)
	WRITE(20,*)'Best result : alph = ',BPAR(4)
	WRITE(20,*)'Best result : beta = ',BPAR(5)
	WRITE(20,*)'Best result : gamm = ',BPAR(6),'  V = ',V3
	PRINT *,'Best result : a =    ',BPAR(1),'  Rp = ',RMIN
	PRINT *,'Best result : b =    ',BPAR(2)
	PRINT *,'Best result : c =    ',BPAR(3)
	PRINT *,'Best result : alph = ',BPAR(4)
	PRINT *,'Best result : beta = ',BPAR(5)
	PRINT *,'Best result : gamm = ',BPAR(6),'  V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
699   CONTINUE
700   IF(NGRID.EQ.0)  GO TO 5000
C
C...  Cell generation by systematic grid
C
C
      IF(NSYS(1).EQ.0)GO TO 1200
C
C    Cubic case
C
      IFILE=1
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Grid search :'
	WRITE(20,*)
	WRITE(20,*)'Results in cubic :'
	WRITE(20,*)'     Rp           Trial number    a     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Grid search :'
	PRINT *
	PRINT *,'Results in cubic :'
	PRINT *,'     Rp           Trial number    a     V'
C
C     READ hkl Miller indices in cub.hkl
C
      TEMPO='cub.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 1101 I=1,NHKL0
1101  READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PMIN-SPAR
1102  CONTINUE
      CELPRE(1)=CELPRE(1)+SPAR
      NTRIED=NTRIED+1
      GO TO 1104
1103  CELPRE(1)=PSTARTB(1)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
1104  CONTINUE
      CELPRE(2)=CELPRE(1)
      CELPRE(3)=CELPRE(1)
      DO 1105 I=1,3
      DO 1105 J=1,3
1105  AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(CELPRE(1).GT.PMAX)GO TO 1116
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 1102
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 1109 I=1,NHKL0
      DO 1106 KK=1,3
1106  HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 1107 J=1,3
      DO 1107 K=J,3
1107  X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 1109
      JH=JH+1
      DO 1108 KK=1,3
1108  JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
1109  CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 1110 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
1110   CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 1111 J=1,NHKL
      FCAL(J)=FOBS(J)
1111  CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 1112 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 1112 J=1,NHKL
      FCAL(J)=FOBS(J)
1112   CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 1114
      CELOLD(1)=CELPRE(1)
      IF(DIFF.GE.RMAX) GO TO 1115
1114  IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 1103
      WRITE(20,*)RMAX,NTRIED,A,V2
      PRINT *,RMAX,NTRIED,A,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=A
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
      CELPRE(1)=CELOLD(1)
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
1115  CONTINUE
C
C... Stop if max limit of grid tests outpassed
C
      IF(CELPRE(1).GT.PMAX)GO TO 1116
      GO TO 1102
1116  CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 1200
      WRITE(20,*)
	WRITE(20,*)'Best result : a=',BPAR(1),'V=',V3,' Rp= ',RMIN
	PRINT *,'Best result : a=',BPAR(1),'V=',V3,' Rp= ',RMIN
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
1200  IF(NSYS(2).EQ.0)GO TO 1300
C
C
C    Hexagonal case
C
C
      IFILE=2
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=120.
	WRITE(20,*)
	WRITE(20,*)'Grid search :'
	WRITE(20,*)
	WRITE(20,*)'Results in hexagonal :'
	WRITE(20,*)'     Rp     Trial number    a      c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Grid search :'
	PRINT *
	PRINT *,'Results in hexagonal :'
	PRINT *,'     Rp     Trial number    a      c     V'
C
C     READ hkl Miller indices in hex.hkl
C
      IF(NSYS(2).EQ.2)GO TO 1260
      TEMPO='hex.hkl'
	GO TO 1261
1260  TEMPO='rho.hkl'
      IFILE=7
1261  CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 1201 I=1,NHKL0
1201  READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PMIN-SPAR
      CELPRE(2)=CELPRE(1)
      CELPRE(3)=PMIN-SPAR
      NTRIED=0
      IFIN=1
C
1202  CONTINUE
C
C     Which parameter to vary ? a or c ?
C
      IF(IFIN.EQ.1)THEN
      CELPRE(1)=CELPRE(1)+SPAR
      IF(CELPRE(1).GT.PMAX)GO TO 1216
	IFIN=0
      NTRIED=NTRIED+1
      ENDIF
      CELPRE(3)=CELPRE(3)+SPAR
      IF(CELPRE(3).GT.PMAX)THEN
      CELPRE(3)=PMIN-SPAR
      IFIN=1
      GO TO 1202
      ENDIF
      NTRIED=NTRIED+1
      GO TO 1204
1203  CONTINUE
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
1204  CONTINUE
      CELPRE(2)=CELPRE(1)
      DO 1205 I=1,3
      DO 1205 J=1,3
1205  AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(CELPRE(1).GT.PMAX)GO TO 1216
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 1202
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 1209 I=1,NHKL0
      DO 1206 KK=1,3
1206  HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 1207 J=1,3
      DO 1207 K=J,3
1207  X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 1209
      JH=JH+1
      DO 1208 KK=1,3
1208  JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
1209  CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 1210 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
1210  CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 1211 J=1,NHKL
      FCAL(J)=FOBS(J)
1211  CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 1212 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 1212 J=1,NHKL
      FCAL(J)=FOBS(J)
1212  CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 1214
      CELOLD(1)=CELPRE(1)
      CELOLD(3)=CELPRE(3)
      IF(DIFF.GE.RMAX)GO TO 1215
1214  IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 1203
      WRITE(20,*)RMAX,NTRIED,A,C,V2
      PRINT *,RMAX,NTRIED,A,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=120.
      NTRIEDB=0
      RMAX=RMAX0
      CELPRE(1)=CELOLD(1)
      CELPRE(3)=CELOLD(3)
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
1215  CONTINUE
C
C... Stop if max limit of grid Carlo tests outpassed
C
      IF(CELPRE(1).GT.PMAX)GO TO 1216
      GO TO 1202
1216  CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 1300
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
1300  IF(NSYS(3).EQ.0)GO TO 1400
C
C    Tetragonal case
C
C
      IFILE=3
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Grid search :'
	WRITE(20,*)
	WRITE(20,*)'Results in tetragonal :'
	WRITE(20,*)'     Rp     Trial number    a      c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Grid search :'
	PRINT *
	PRINT *,'Results in tetragonal :'
	PRINT *,'     Rp     Trial number    a      c     V'
C
C     READ hkl Miller indices in tet.hkl
C
      TEMPO='tet.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 1301 I=1,NHKL0
1301  READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PMIN-SPAR
      CELPRE(2)=CELPRE(1)
      CELPRE(3)=PMIN-SPAR
      NTRIED=0
      IFIN=1
C
1302  CONTINUE
C
C     Which parameter to vary ? a or c ?
C
      IF(IFIN.EQ.1)THEN
      CELPRE(1)=CELPRE(1)+SPAR
      IF(CELPRE(1).GT.PMAX)GO TO 1316
	IFIN=0
      NTRIED=NTRIED+1
      ENDIF
      CELPRE(3)=CELPRE(3)+SPAR
      IF(CELPRE(3).GT.PMAX)THEN
      CELPRE(3)=PMIN-SPAR
      IFIN=1
      GO TO 1302
      ENDIF
      NTRIED=NTRIED+1
      GO TO 1304
1303  CONTINUE
      I=3
      IF(RAN(ISEED).GT.0.5)I=1
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
1304  CONTINUE
      CELPRE(2)=CELPRE(1)
      DO 1305 I=1,3
      DO 1305 J=1,3
1305  AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(CELPRE(1).GT.PMAX)GO TO 1316
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	GO TO 1302
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 1309 I=1,NHKL0
      DO 1306 KK=1,3
1306  HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 1307 J=1,3
      DO 1307 K=J,3
1307  X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 1309
      JH=JH+1
      DO 1308 KK=1,3
1308  JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
1309  CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 1310 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
1310  CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 1311 J=1,NHKL
      FCAL(J)=FOBS(J)
1311  CONTINUE
C
C...  Three more iterations by Le Bail fit
C
      DO 1312 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 1312 J=1,NHKL
      FCAL(J)=FOBS(J)
1312  CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 1314
      CELOLD(1)=CELPRE(1)
      CELOLD(3)=CELPRE(3)
      IF(DIFF.GE.RMAX)GO TO 1315
1314  IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 1303
      WRITE(20,*)RMAX,NTRIED,A,C,V2
      PRINT *,RMAX,NTRIED,A,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=A
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
      CELPRE(1)=CELOLD(1)
      CELPRE(3)=CELOLD(3)
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
1315  CONTINUE
C
C... Stop if max limit of grid tests outpassed
C
      IF(CELPRE(1).GT.PMAX)GO TO 1316
      GO TO 1302
1316  CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 1400
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
1400  IF(NSYS(4).EQ.0)GO TO 1500
C
C    Orthorhombic case
C
C
      IFILE=4
      NCYCLES=400
      CELPRE(4)=90.
      CELPRE(5)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Grid search :'
	WRITE(20,*)
	WRITE(20,*)'Results in orthorhombic :'
	WRITE(20,*)'     Rp     Trial number    a   b   c     V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Grid search :'
	PRINT *
	PRINT *,'Results in orthorhombic :'
	PRINT *,'     Rp     Trial number    a   b   c     V'
C
C     READ hkl Miller indices in ort.hkl
C
      TEMPO='ort.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 1401 I=1,NHKL0
1401  READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PMIN-SPAR
      CELPRE(2)=PMIN-SPAR
      CELPRE(3)=PMIN-SPAR
      NTRIED=0
      IFIN=1
      IFIN2=1
C
1402  CONTINUE
C
C     Which parameter to vary ? a or b or c ?
C
      IF(IFIN.EQ.1)THEN
      CELPRE(1)=CELPRE(1)+SPAR
	PRINT *,'  a = ',CELPRE(1)
      IF(CELPRE(1).GT.PMAX)GO TO 1416
	IFIN=0
      NTRIED=NTRIED+1
      ENDIF
      IF(IFIN2.EQ.1)THEN
      CELPRE(2)=CELPRE(2)+SPAR
	IFIN2=0
      ENDIF
	IF(CELPRE(2).GT.PMAX)THEN
      CELPRE(2)=PMIN-SPAR
	IFIN=1
      GO TO 1402
      ENDIF
      NTRIED=NTRIED+1
      CELPRE(3)=CELPRE(3)+SPAR
      IF(CELPRE(3).GT.PMAX)THEN
      CELPRE(3)=PMIN-SPAR
      IFIN2=1
      GO TO 1402
      ENDIF
      NTRIED=NTRIED+1
      GO TO 1404
1403  CONTINUE
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.33333)I=1
      IF(X.GE.0.33333.AND.X.LT.0.66666)I=2
      IF(X.GE.0.66666.AND.X.LE.1.)I=3
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      NTRIEDB=NTRIEDB+1
1404  CONTINUE
      DO 1405 I=1,3
      DO 1405 J=1,3
1405  AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(CELPRE(1).GT.PMAX)GO TO 1416
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 1402
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 1409 I=1,NHKL0
      DO 1406 KK=1,3
1406  HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 1407 J=1,3
      DO 1407 K=J,3
1407  X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 1409
      JH=JH+1
      DO 1408 KK=1,3
1408  JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
1409  CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 1410 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
1410  CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 1411 J=1,NHKL
      FCAL(J)=FOBS(J)
1411  CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 1412 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 1412 J=1,NHKL
      FCAL(J)=FOBS(J)
1412  CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 1414
      CELOLD(1)=CELPRE(1)
      CELOLD(2)=CELPRE(2)
      CELOLD(3)=CELPRE(3)
      IF(DIFF.GE.RMAX)GO TO 1415
1414  IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
      B=CELPRE(2)
	C=CELPRE(3)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	BPAR(2)=B
      BPAR(3)=C
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(2)=CELPRE(2)
      PSTARTB(3)=CELPRE(3)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 1403
      WRITE(20,*)RMAX,NTRIED,A,B,C,V2
      PRINT *,RMAX,NTRIED,A,B,C,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=B
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=90.
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
      CELPRE(1)=CELOLD(1)
      CELPRE(2)=CELOLD(2)
      CELPRE(3)=CELOLD(3)
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
1415  CONTINUE
C
C... Stop if max limit of grid tests outpassed
C
      IF(CELPRE(1).GT.PMAX)GO TO 1416
      GO TO 1402
1416  CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 1500
      WRITE(20,*)
	WRITE(20,*)'Best result : a = ',BPAR(1),' Rp = ',RMIN
	WRITE(20,*)'Best result : b = ',BPAR(2)
	WRITE(20,*)'Best result : c = ',BPAR(3),'V = ',V3
	PRINT *,'Best result : a = ',BPAR(1),' Rp = ',RMIN
	PRINT *,'Best result : b = ',BPAR(2)
	PRINT *,'Best result : c = ',BPAR(3),'V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
1500  IF(NSYS(5).EQ.0)GO TO 1600
C
C    Monoclinic case - would be too long in grid search, but...
C
C
      IFILE=5
      NCYCLES=800
      CELPRE(4)=90.
      CELPRE(6)=90.
	WRITE(20,*)
	WRITE(20,*)'Grid search :'
	WRITE(20,*)
	WRITE(20,*)'Results in monoclinic :'
	WRITE(20,*)'     Rp     Trial number    a   b   c  bet  V'
	WRITE(20,*)
	PRINT *
	PRINT *,'Grid search :'
	PRINT *
	PRINT *,'Results in monoclinic :'
	PRINT *,'     Rp     Trial number    a   b   c  bet  V'
C
C     READ hkl Miller indices in mon.hkl
C
      TEMPO='mon.hkl'
      CALL OPEN_READ1(35,TEMPO)
      READ(35,*)NHKL0
      DO 1501 I=1,NHKL0
1501  READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
C
C...  here starts the loop
C
      CELPRE(1)=PMIN-SPAR
      CELPRE(2)=PMIN-SPAR
      CELPRE(3)=PMIN-SPAR
      CELPRE(5)=90.-SANG
      NTRIED=0
      IFIN1=1
      IFIN2=1
      IFIN3=1
C
1502  CONTINUE
C
C     Which parameter to vary ? a or b or c or bet ?
C
      IF(IFIN1.EQ.1)THEN
      CELPRE(1)=CELPRE(1)+SPAR
	PRINT *,'  a = ',CELPRE(1)
      IF(CELPRE(1).GT.PMAX)GO TO 1516
	IFIN1=0
      NTRIED=NTRIED+1
      ENDIF
      IF(IFIN2.EQ.1)THEN
      CELPRE(2)=CELPRE(2)+SPAR
	IFIN2=0
      ENDIF
	IF(CELPRE(2).GT.PMAX)THEN
      CELPRE(2)=PMIN-SPAR
	IFIN1=1
      GO TO 1502
      ENDIF
      NTRIED=NTRIED+1
      IF(IFIN3.EQ.1)THEN
      CELPRE(3)=CELPRE(3)+SPAR
      IFIN3=0
      ENDIF
      IF(CELPRE(3).GT.PMAX)THEN
      CELPRE(3)=PMIN-SPAR
      IFIN2=1
      GO TO 1502
      ENDIF
      NTRIED=NTRIED+1
      CELPRE(5)=CELPRE(5)+SANG
      IF(CELPRE(5).GT.130.)THEN
      CELPRE(5)=90.-SANG
      IFIN3=1
      GO TO 1502
      ENDIF
      NTRIED=NTRIED+1
      GO TO 1504
1503  CONTINUE
      X=RAN(ISEED)
      IF(X.GE.0.AND.X.LT.0.25)I=1
      IF(X.GE.0.25.AND.X.LT.0.5)I=2
      IF(X.GE.0.5.AND.X.LT.0.75)I=3
      IF(X.GE.0.75.AND.X.LE.1.0)I=5
      IF(I.NE.5)THEN
      CELPRE(I)=PSTARTB(I)+(DELTAB*(RAN(ISEED)-0.5)*2.)
      ELSE
      CELPRE(I)=PSTARTB(I)+(DELTAD*(RAN(ISEED)-0.5)*2.)
      ENDIF
      NTRIEDB=NTRIEDB+1
1504  CONTINUE
C      WRITE(20,*)CELPRE(1),CELPRE(2),CELPRE(3),CELPRE(5)
C      WRITE(*,*)CELPRE(1),CELPRE(2),CELPRE(3),CELPRE(5)
      DO 1505 I=1,3
      DO 1505 J=1,3
1505  AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
      IF(CELPRE(1).GT.PMAX)GO TO 1516
	IF(V1.GT.VMAX.OR.V1.LT.VMIN) THEN
	NTRIED=NTRIED-1
	GO TO 1502
	ENDIF
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 1509 I=1,NHKL0
      DO 1506 KK=1,3
1506  HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 1507 J=1,3
      DO 1507 K=J,3
1507  X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 1509
      JH=JH+1
      DO 1508 KK=1,3
1508  JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
1509  CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 1510 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
1510  CONTINUE
C
C...  First iteration by Le Bail fit
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 1511 J=1,NHKL
      FCAL(J)=FOBS(J)
1511  CONTINUE
C
C...  NLB+1 more iterations by Le Bail fit
C
      DO 1512 K=1,NLB
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 1512 J=1,NHKL
      FCAL(J)=FOBS(J)
1512  CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
C
C... Rp value satisfying ???
C
      IF(NTRIEDB.NE.0)GO TO 1514
      CELOLD(1)=CELPRE(1)
      CELOLD(2)=CELPRE(2)
      CELOLD(3)=CELPRE(3)
      CELOLD(5)=CELPRE(5)
      IF(DIFF.GE.RMAX)GO TO 1515
1514  IF(DIFF.LE.RMAX)THEN
      RMAX=DIFF
      A=CELPRE(1)
      B=CELPRE(2)
	C=CELPRE(3)
	BET=CELPRE(5)
	V2=V1
	IF(DIFF.LT.RMIN)THEN
	RMIN=DIFF
	BPAR(1)=A
	BPAR(2)=B
      BPAR(3)=C
      BPAR(5)=BET
	V3=V1
	ENDIF
C
C... "Refine" that cell (by Monte Carlo too...)
C
      PSTARTB(1)=CELPRE(1)
      PSTARTB(2)=CELPRE(2)
      PSTARTB(3)=CELPRE(3)
      PSTARTB(5)=CELPRE(5)
	ENDIF
      IF(NTRIEDB.LE.NCYCLES)GO TO 1503
      WRITE(20,*)RMAX,NTRIED,A,B,C,BET,V2
      PRINT *,RMAX,NTRIED,A,B,C,BET,V2
      IGC=IGC+1
      IF(IGC.GT.1000)THEN
      WRITE(20,*)'   More than 1000 good cells = STOP'
      PRINT *,'   More than 1000 good cells = STOP'
      GO TO 5000
      ENDIF
	IFI(IGC)=IFILE
      VGC(IGC)=V2
      RP(IGC)=RMAX
      CEL(1,IGC)=A
      CEL(2,IGC)=B
      CEL(3,IGC)=C
      CEL(4,IGC)=90.
      CEL(5,IGC)=BET
      CEL(6,IGC)=90.
      NTRIEDB=0
      RMAX=RMAX0
      CELPRE(1)=CELOLD(1)
      CELPRE(2)=CELOLD(2)
      CELPRE(3)=CELOLD(3)
      CELPRE(5)=CELOLD(5)
	IF(RP(IGC).LT.RMI)THEN
      WRITE(20,*)
	WRITE(20,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
      WRITE(*,*)
	WRITE(*,*)' YOU HAVE FOUND AN INTERESTING RESULT : Rp < Rmin!'
	GO TO 5000
	ENDIF
1515  CONTINUE
C
C... Stop if max limit of grid tests outpassed
C
      IF(CELPRE(1).GT.PMAX)GO TO 1516
      GO TO 1502
1516  CONTINUE
      IF(RMIN.EQ.RMAX)GO TO 1600
      WRITE(20,*)
	WRITE(20,*)'Best result : a =   ',BPAR(1),'  Rp = ',RMIN
	WRITE(20,*)'Best result : b =   ',BPAR(2)
	WRITE(20,*)'Best result : c =   ',BPAR(3)
	WRITE(20,*)'Best result : bet = ',BPAR(5),'  V = ',V3
	PRINT *,'Best result : a =   ',BPAR(1),'  Rp = ',RMIN
	PRINT *,'Best result : b =   ',BPAR(2)
	PRINT *,'Best result : c =   ',BPAR(3)
	PRINT *,'Best result : bet = ',BPAR(5),'  V = ',V3
	WRITE(20,*)
      RMIN=RMAX
      CALL DATN
C
C
1600  IF(NSYS(6).EQ.0)GO TO 5000
C
C    Triclinic case - would be too long in grid search...
C
C
5000  CONTINUE
      IF(IGC.EQ.0)GO TO 6000
C
C   Prepare sorted output for CHEKCELL
C
      TEMPO=FILE(1:LFILE)//'.ckm'
      INQUIRE(FILE=TEMPO,EXIST=QEX)
      IF(.NOT.QEX) GO TO 1998
      CALL FILEDEL(24,TEMPO)
1998  CALL OPEN_WRITE1(24,TEMPO)
      CALL SORT(IGC,RP,LL)
      WRITE(20,1999)
1999  FORMAT('    RP     Vol     Vol/V1        a        b     
     1    c      alpha  beta  gamma')
      KK=20
      DO 2000 I=1,IGC
      J=LL(I)
      X=1./RP(J)*5.
      VR=VGC(J)/VGC(LL(1))
      WRITE(20,2001)RP(J),VGC(J),VR,(CEL(K,J),K=1,6)
      WRITE(24,2002)KK,X,VGC(J),VR,(CEL(K,J),K=1,6)
2000  CONTINUE
2001  FORMAT(F8.3,F11.3,F6.2,4X,3F9.4,3F8.3)
2002  FORMAT(I2,F8.2,F11.3,F6.2,39X,3F9.4,3F8.3)
      CLOSE(24)
C
C   Make a .prf with the best solution
C
      DO 2030 I=1,6
2030  CELPRE(I)=CEL(I,LL(1))
C
C     READ hkl Miller indices in cub.hkl
C
      IFILE=IFI(LL(1))
      GO TO (31,32,33,34,35,36,37)IFILE
31    TEMPO='cub.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=400
      DO 41 I=1,NHKL0
41    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
32    TEMPO='hex.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=800
      DO 42 I=1,NHKL0
42    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
33    TEMPO='tet.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=800
      DO 43 I=1,NHKL0
43    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
34    TEMPO='ort.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=1000
      DO 44 I=1,NHKL0
44    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
35    TEMPO='mon.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=1000
      DO 45 I=1,NHKL0
45    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
36    TEMPO='tri.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=1000
      DO 46 I=1,NHKL0
46    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
37    TEMPO='rho.hkl'
      CALL OPEN_READ1(35,TEMPO)
      NHKL0=600
      DO 47 I=1,NHKL0
47    READ(35,*)(IHH(KK,I),KK=1,3)
      CLOSE(35)
	GO TO 50
C
C
50    CONTINUE
      DO 55 I=1,3
      DO 55 J=1,3
55    AL(I,J)=0.0
      CALL DCELL(CELPRE,AL,RCELLN,V1)
C
C...  Keep only the hkl for d > dmin
C
      JH=0
      DO 59 I=1,NHKL0
      DO 56 KK=1,3
56    HH(KK)=IHH(KK,I)
C     HH IS INDICES OF REFLECTION
      X=0.
      DO 57 J=1,3
      DO 57 K=J,3
57    X=AL(J,K)*HH(J)*HH(K)+X
      X=1/SQRT(X)
      IF(X.LT.DMIN)GO TO 59
      JH=JH+1
      DO 58 KK=1,3
58    JHH(KK,JH)=IHH(KK,I)
      FCAL(JH)=50.
      D(JH)=X
C     X IS D(hkl) FOR REFLECTION HH
59    CONTINUE
      NHKL=JH
C
C   Again, calculate 2-theta, etc.
C
      DO 60 NM=1,NHKL
C
      DD=D(NM)
      SSS=1/DD**2
C
C     Needs to calculate halfwidths and positions again
C
      SINTH=SLABDA*SLABDA*SSS/4.
      COSTH=1.-SINTH
      TANTH=SQRT(SINTH/COSTH)
      THETA(NM)=ATAN(TANTH)*PI
      HW(NM)=SQRT(U*TANTH*TANTH+V*TANTH+W)
      IF(HMIN.GT.HW(NM))HMIN=HW(NM)
      HW(NM)=HW(NM)*SLOG
      HW4(NM)=HW(NM)/SLOG*2.0
      BBB(NM)=HW(NM)*HW(NM)
60    CONTINUE
C
C...  Calculate best Yobs
C
      CALL PROFILE1(NHKL,FCAL,FOBS)
      DO 61 J=1,NHKL
      FCAL(J)=FOBS(J)
61    CONTINUE
C
C...  6 more iterations by Le Bail fit
C
CCC      DO 62 K=1,6
	CALL PROFILE2(NHKL,FCAL,FOBS)
      DO 62 J=1,NHKL
      FCAL(J)=FOBS(J)
62    CONTINUE
      CALL PROFILE3(NHKL,FCAL,DIFF)
      DIFF=DIFF/SUM_Y
	WRITE(20,*)
	WRITE(20,*)' Final Rp on the best solution = ',DIFF
	WRITE(20,*)
C
C     Make the .prf
C
      TEMPO=FILE(1:LFILE)//'.prf'
      INQUIRE(FILE=TEMPO,EXIST=QEX)
      IF(.not.QEX) go to 2050
      CALL FILEDEL(12,TEMPO)
2050  CALL OPEN_WRITE1(12,TEMPO)
      IVERS=8
      ZERO=0.
      WRITE(12,'(20A4)')(TEXT(I),I=1,20)
      WRITE(12,'(A,F9.4)') '3111   1.0000',ZERO
      THMAX=POS(N2)
      THMIN=POS(N1)
      ICN=NHKL
      DO 2051 I=1,NHKL
      I1=JHH(1,I)
      I2=JHH(2,I)
      I3=JHH(3,I)
2051  IREFS(I)=256*(256*(256*(8+1)+128+I1)+128+I2)+128+I3
      ICZ=NHKL
      NPTS=N2-N1+1
      WRITE(12,'(2F8.3,F8.5,I8)') THMAX,THMIN,STEP,IVERS
      AMDA1=SLABDA
      AMDA2=SLABDA
      NPAT1=1
      NVK=0
      WRITE(12,'(I3,I7,5f10.5)')NPAT1,NPTS,AMDA1,AMDA2,ZERO
      WRITE(12,'(8I5,8i3)')ICN,NVK
      WRITE(12,'(8(F7.0,1X))') (YOBS(J),J=N1,N2)
      WRITE(12,'(8(F7.0,1X))') (YCALC(J),J=N1,N2)
      WRITE(12,'(8I10)') (IREFS(J),J=1,ICZ)
      WRITE(12,'(10F8.3)')(THETA(J),J=1,ICZ)
      NEXCRG=0
      EXCRG=0
6000  WRITE(12,'(I8)') NEXCRG
      WRITE(12,'(2F8.2)') EXCRG,EXCRG
      CLOSE(12)
C
C
C
3000  FORMAT(/1x,'=============================================
     -===================='/
     -        1x,' McMaille version ',a,' by A. Le Bail - 2002 -
     - alb@cristal.org'/
     -        1x,'=============================================
     -===================='//
     -        1x,'Using generic filename : ',A/)
      GO TO 3600
3500  WRITE(20,*)'  Error reading the first line : TEXT'
	WRITE (BUFFER,*)'  Error reading the first line : TEXT'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3501  WRITE(20,*)'  Error reading lambda, etc, line 2'
	WRITE (BUFFER,*)'  Error reading lambda, etc, line 2'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3502  WRITE(20,*)'  Error reading symmetry code, line 3'
	WRITE (BUFFER,*)'  Error reading symmetry code, line 3'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3503  WRITE(20,*)'  Error reading U, V, W, step'
	WRITE (BUFFER,*)'  Error reading U, V, W, Step'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3504  WRITE(20,*)'  Error reading Pmin, Pmax, Vmin'
	WRITE (BUFFER,*)'  Error reading Pmin, Pmax, Vmin'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3505  WRITE(20,*)'  Error reading grid steps'
	WRITE (BUFFER,*)'  Error reading grid steps'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3506  WRITE(20,*)'  Error reading ntimelim'
	WRITE (BUFFER,*)'  Error reading timelim'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3507  WRITE(20,*)'  Error reading nstart, rmax, test'
	WRITE (BUFFER,*)'  Error reading nstart, rmax, test'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
      GO TO 3600
3508  WRITE(20,*)'  Error reading data angle > 180'
	WRITE (BUFFER,*)'  Error reading data angle > 180'
	CALL PROGRESSVIEW (PROGRESS, BUFFER)
3600  CONTINUE
      CALL DATN
      WRITE(*,3955)
3955  FORMAT(//'  Type any character and a RETURN to continue : ',$)
      READ(*,*)FEND
      CLOSE(20)
      CLOSE(28)
      STOP
      END
*-----------------------------------------------------------------------
	SUBROUTINE PROGRESSVIEW(PROGRESS,BUFFER)
	INTEGER(4) PROGRESS
	CHARACTER*79 BUFFER
      WRITE(*,1)BUFFER
1     FORMAT(A79)
      RETURN
      END
*-----------------------------------------------------------------------
      SUBROUTINE DATN
C
C Obviously, this subroutine is compiler specific...
C
C In case of problem, remove that subroutine and all 
C the CALL DATN inside the program.
C
C Date will be printed as follows :
C
C     10-Apr-2000   Hour:  0 Min: 29 Sec: 20 
C
C     dd-mmm-yyyy          hh     mm      ss 
C
C     DT_VALUES(3),MONTHS(DT_VALUES(2)),DT_VALUES(1), 
C     DT_VALUES(5),DT_VALUES(6),DT_VALUES(7) 
C
C
      CHARACTER*8 DA
      CHARACTER*10 TIME
      CHARACTER*5 ZONE
      INTEGER*4 DT_VALUES(8)    ! Return values from DATE_AND_TIME
      CHARACTER*3 MONTHS(12)    ! Month names
      DATA MONTHS /'Jan','Feb','Mar','Apr','May','Jun',
     1             'Jul','Aug','Sep','Oct','Nov','Dec'/

C
C Get current date values
C
       CALL DATE_AND_TIME (DA,TIME,ZONE,DT_VALUES) 
C
C Format date
C
      WRITE(20,*)
      WRITE(20,10) DT_VALUES(3),
     1 MONTHS(DT_VALUES(2)),DT_VALUES(1),DT_VALUES(5),
     2 DT_VALUES(6),DT_VALUES(7)
      WRITE(20,*)
10    FORMAT(1X,I2,'-',A3,'-',I4,5X,I2,' hour ',I2,' min ',I2,
     -' Sec ')
C
C  End of compiler specific subroutine
C
      RETURN
      END
      SUBROUTINE OPEN_READ1(UNIT,FILE)
      integer UNIT
      CHARACTER*(*) FILE
      OPEN (UNIT,FILE=FILE,STATUS='old')
      RETURN
      END
*
      SUBROUTINE OPEN_WRITE1(UNIT,FILE)
      integer UNIT
      CHARACTER*(*) FILE
      OPEN (UNIT,FILE=FILE,STATUS='NEW')
      RETURN
      END
*
      SUBROUTINE FILEDEL(UNIT,FILE)
      integer UNIT
      CHARACTER*(*) FILE
      OPEN (UNIT,FILE=FILE,STATUS='old')
      close(UNIT,STATUS='delete')
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
*     SUBROUTINE OPEN_WRITE OPENs a FILE for writing, filling in the 
*     supplied extension IF none is supplied with the FILE name.
*     Allows use of system specIFic facilities of OPEN statement.
*
      SUBROUTINE OPEN_WRITE(UNIT,FILE,extension)
      integer UNIT
      CHARACTER*(*) FILE,extension,temp*80
      temp=FILE
      i=index(FILE,'.')
      l=LEN(FILE)
      DO WHILE(FILE(l:l).EQ.' ')
         l=l-1
      ENDDO
      IF (i.EQ.0) temp=FILE(1:l)//extension
      OPEN (UNIT,FILE=temp,STATUS='old',err=10)
      close(UNIT,STATUS='delete')
10    OPEN (UNIT,FILE=temp,STATUS='NEW')
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
*     SUBROUTINE ESP_INIT must set the seed for the random number 
*     generator and obtain the current cpu clock reading in minutes 
*
      SUBROUTINE ESP_INIT(ISEED,TIME_START) 
C
C PORTLIB is compiler spcecific part, introduced for 
C using the intrinsic function SECNDS(X) which returns the 
C  (time in seconds since midnight - X) 
C
C      USE PORTLIB
      USE DFPORT
      X=0.
      ISEED=INT(GSECNDS(X))*2+1
      TIME_START=0.
*
*  run the random number generator N times
*  for avoiding effects of starting value
*
      N=ISEED/2000
      IF(N.LE.0)N=100
      DO 10 I=1,N
      BIDON=RAN(ISEED)
10    ISEED=ISEED/3
      ISEED=ISEED*2+1
      WRITE(20,*)' ISEED = ',ISEED
      RETURN
      END
*
*----------------------------------------------------------------------- 
      function gsecnds(sek)
! machine dependent SECNDS
! replace this line by your specific call
C
C PORTLIB is compiler spcecific part, introduced for 
C using the intrinsic function SECNDS(X) which returns the 
C  (time in seconds since midnight - X) 
C      USE PORTLIB
      USE DFPORT
      gsecnds=SECNDS(sek)
      return
      end
*
*********************************************************************** 
*
      SUBROUTINE PROFILE0(NHKL,F)
      INCLUDE 'esp0.inc'
      INCLUDE 'esp2.inc'
      DIMENSION F(N_HKL)
*  Sum at each point
      DO 191 K=1,NPTS
      YOBS(K)=0.
*  SUM on all hkl
      DO 190 J=1,NHKL
      DELTA=POS(K)-THETA(J)
      IF(ABS(DELTA).GT.HW4(J))GO TO 190
      DELT=DELTA*DELTA
      OMEG=EXP(-DELT/BBB(J))/HW(J)
      YOBS(K)=YOBS(K)+OMEG*F(J)
190   CONTINUE
      IF(YOBS(K).LT.0.01)YOBS(K)=0.
191   CONTINUE
      L=NPTS+1
      DO 194 K=1,NPTS
      L=L-1
194   IF(YOBS(L).NE.0.)GO TO 195
195   N2=L
      RETURN
      END
*
*********************************************************************** 
*
      SUBROUTINE PROFILE1(NHKL,F,FOBS)
      INCLUDE 'esp0.inc'
      INCLUDE 'esp2.inc'
      DIMENSION F(N_HKL),FOBS(N_HKL),DUMP(N_HKL)
	DIMENSION SOMEGA(N_HKL)
*  Sum at each point
      DO 189 K=1,NHKL
	SOMEGA(K)=0.
189   FOBS(K)=0.
      DO 191 K=1,N2
      YCALC(K)=0.
*  SUM on all hkl
      KPOS=0
      OMEGT=0.
      DO 190 J=1,NHKL
      DELTA=POS(K)-THETA(J)
      IF(ABS(DELTA).GT.HW4(J))GO TO 190
      IF(KPOS.EQ.0)NHA(K)=J
      KPOS=1
      NHB(K)=J
      DELT=DELTA*DELTA
      OMEG=EXP(-DELT/BBB(J))/HW(J)
	SOMEGA(J)=SOMEGA(J)+OMEG
      DUMP(J)=F(J)*OMEG
      YCALC(K)=YCALC(K)+DUMP(J)
190   CONTINUE
      IF(YCALC(K).EQ.0.)GO TO 230
      YOY=YOBS(K)/YCALC(K)
      DO 193 J=NHA(K),NHB(K)
      FOBS(J)=FOBS(J)+DUMP(J)*YOY
193   CONTINUE
230   CONTINUE
191   CONTINUE
      DO 200 K=1,NHKL
	IF(SOMEGA(K).EQ.0.)GO TO 220
      FOBS(K)=FOBS(K)/SOMEGA(K)
	GO TO 200
220   FOBS(K)=0.
200   CONTINUE
      RETURN
      END
*
*********************************************************************** 
*
      SUBROUTINE PROFILE2(NHKL,F,FOBS)
      INCLUDE 'esp0.inc'
      INCLUDE 'esp2.inc'
      DIMENSION F(N_HKL),FOBS(N_HKL),DUMP(N_HKL)
	DIMENSION SOMEGA(N_HKL)
*  Sum at each point
      DO 189 K=1,NHKL
	SOMEGA(K)=0.
189   FOBS(K)=0.
      DO 191 K=1,N2
      YCALC(K)=0.
*  SUM on all hkl
      OMEGT=0.
      DO 190 J=NHA(K),NHB(K)
      DELTA=POS(K)-THETA(J)
      IF(ABS(DELTA).GT.HW4(J))GO TO 190
      DELT=DELTA*DELTA
      OMEG=EXP(-DELT/BBB(J))/HW(J)
	SOMEGA(J)=SOMEGA(J)+OMEG
      DUMP(J)=F(J)*OMEG
      YCALC(K)=YCALC(K)+DUMP(J)
190   CONTINUE
      IF(YCALC(K).EQ.0.)GO TO 230
      YOY=YOBS(K)/YCALC(K)
      DO 193 J=NHA(K),NHB(K)
      FOBS(J)=FOBS(J)+DUMP(J)*YOY
193   CONTINUE
230   CONTINUE
191   CONTINUE
      DO 200 K=1,NHKL
	IF(SOMEGA(K).EQ.0.)GO TO 220
      FOBS(K)=FOBS(K)/SOMEGA(K)
	GO TO 200
220   FOBS(K)=0.
200   CONTINUE
      RETURN
      END
*
*********************************************************************** 
*
*     Profile Ycalc only at the last cycle for obtaining Rp
*
      SUBROUTINE PROFILE3(NHKL,F,DIFF)
      INCLUDE 'esp0.inc'
      INCLUDE 'esp2.inc'
      DIMENSION F(N_HKL)
	DIFF=0.
*  Sum at each point
      DO 191 K=1,N2
      YCALC(K)=0.
      IF(YOBS(K).EQ.0.)GO TO 191
*  SUM on all hkl
      OMEGT=0.
      DO 190 J=NHA(K),NHB(K)
      DELTA=POS(K)-THETA(J)
      IF(ABS(DELTA).GT.HW4(J))GO TO 190
      DELT=DELTA*DELTA
      OMEG=EXP(-DELT/BBB(J))/HW(J)
      YCALC(K)=YCALC(K)+F(J)*OMEG
190   CONTINUE
191   CONTINUE
      DO 192 K=1,N2
      DIFF=DIFF+ABS(YOBS(K)-YCALC(K))
192   CONTINUE
      RETURN
      END
*
*********************************************************************** 
*
      SUBROUTINE DCELL(CELLN,AL,RCELLN,V)
      DIMENSION CELLN(6),AL(3,3),RCELLN(6),CELL(6)
      DEGRAD=3.14159/180.0
      DO 30 I=1,6
30    CELL(I)=CELLN(I)
      DO 31 I=1,3
      L=I+3
      IF(CELL(L)-90.0)32,33,32
33    CELL(L)=0.0
      GOTO 31
32    CELL(L)=COS(DEGRAD*CELL(L))
31    CONTINUE
      CALL TRCL(CELL,RCELLN,V)
C     RCELL IS THE RECIPROCAL CELL CONSTANTS
      DO 34 I=1,3
      AL(I,I)=RCELLN(I)*RCELLN(I)
      CALL PERM(I,J,K)
      IF(J-K)35,36,36
35    AL(J,K)=2.0*RCELLN(J)*RCELLN(K)*RCELLN(I+3)
      GOTO 34
36    AL(K,J)=2.0*RCELLN(J)*RCELLN(K)*RCELLN(I+3) 
34    CONTINUE
      DO 37 I=4,6
37    RCELLN(I)=ACOS(RCELLN(I))/DEGRAD
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
      SUBROUTINE TRCL(CELLN,RCELLN,V)
C     TRANSFORMS REAL CELL TO RECIPROCAL OR VICE VERSA 
C     INPUT CELL IS IN ARRAY CELL AS LENGTHS AND COSINES
      DIMENSION CELLN(6),RCELLN(6)
      DIMENSION SINA(3)
      ABC=1.0
      PROD=2.0
      V=-2.0
      DO 10 I=1,3
      L=I+3
      SINA(I)=1.0-CELLN(L)**2
      V=V+SINA(I)
      SINA(I)=SQRT(SINA(I))
      PROD=PROD*CELLN(L)
10    ABC=ABC*CELLN(I)
      V=ABC*SQRT(V+PROD)
C      V IS CELL VOLUME
C     PUT INVERTED CELL INTO RCELL
      DO 20 I=1,3
      CALL PERM(I,J,K)
      RCELLN(I)=CELLN(J)*CELLN(K)*SINA(I)/V
      L=I+3
20    RCELLN(L)=(CELLN(J+3)*CELLN(K+3)-CELLN(L))/(SINA(J)
     2 *SINA(K))
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
      SUBROUTINE PERM(I,J,K)
C     PERMS USEFUL COMBINATIONS OF INTEGERS IN THE RANGE 1 TO 3
      IF(I-2)10,20,30
10    J=2
      K=3
      RETURN
20    J=3
      K=1
      RETURN
30    J=1
      K=2
      RETURN
      END
*
********************************************************* 
*
      SUBROUTINE SORT(N,A,L)
C     ********************************************************
C     THE SUBROUTINE SORT APPLIES TO THE ARRAY A WITH J ELEMENTS.
C     THE INDICES OF THE ORDERED ARRAY ARE PLACED IN ARRAY L
C     TO OBTAIN THE ORDERED ARRAY REPLACE THE INDICES I OF A(I)
C     WITH I=L(I)---REF.CACM 271
C     *********************************************************
C
      DIMENSION IUT(10),ILT(10)
C
      DIMENSION A(1000),L(1000)
      J=N
      I=1
      M=1
      DO 10 K=1,J
10    L(K)=K
20    IF(J-I-1)140,140,30
30    IP=(J+I)/2
      ITT=L(IP)
      T=A(ITT)
      L(IP)=L(I)
      IQ=J
      K=I+1
40    IF(K.GT.IQ)GOTO 90
      LK=L(K)
      IF(A(LK).LE.T)GOTO 80
      IQ=IQ
50    IF(IQ.LT.K)GOTO 70
      LQ=L(IQ)
      IF(A(LQ).GE.T)GOTO 60
      IX=L(K)
      L(K)=L(IQ)
      L(IQ)=IX
      IQ=IQ-1
      GOTO 80
60    IQ=IQ-1
      GOTO 50
70    CONTINUE
      IQ=K-1
      GOTO 100
80    K=K+1
      GOTO 40
90    CONTINUE
100   L(I)=L(IQ)
      L(IQ)=ITT
      IF(2*IQ-I-J)120,120,110
110   ILT(M)=I
      IUT(M)=IQ-1
      I=IQ+1
      GOTO 130
120   ILT(M)=IQ+1
      IUT(M)=J
      J=IQ-1
130   M=M+1
      GOTO 20
140   IF(I-J)150,170,170
150   LI=L(I)
      LJ=L(J)
      IF(A(LI)-A(LJ))170,170,160
160   IX=L(I)
      L(I)=L(J)
      L(J)=IX
170   M=M-1
      IF(M)190,190,180
180   I=ILT(M)
      J=IUT(M)
      GOTO 20
190   CONTINUE
      RETURN
      END
*

